/***************************************************************************
* Copyright (c) 2000-2004, Future Systems, Inc. / Seoul, Republic of Korea *
* All Rights Reserved.                                                     *
*                                                                          *
* This document contains proprietary and confidential information.  No     *
* parts of this document or the computer program it embodies may be in     *
* any way copied, duplicated, reproduced, translated into a different      *
* programming language, or distributed to any person, company, or          *
* corporation without the prior written consent of Future Systems, Inc.    *
*                              Hyo Sun Hwang                               *
*                372-2 YangJae B/D 6th Floor, Seoul, Korea                 *
*                           +82-2-578-0581 (552)                           *
***************************************************************************/

/*--------------------- [ Version/Command in detais] ---------------------*\
Description : filename.? (CrypLib version 1.00)
			(head file)	(C-source file)

C0000 : Created by Hyo Sun Hwang (hyosun@future.co.kr) 2000/12/31

C0001 : Modified by Hyo Sun Hwang (hyosun@future.co.kr) 2001/01/01

\*------------------------------------------------------------------------*/

/*************** Header files *********************************************/
#include <stdio.h>
#include <time.h>
#include "sha256.h"

/*************** Assertions ***********************************************/

/*************** Definitions / Macros  ************************************/
#define KILO	1024
#define MEGA	(KILO*KILO)

////	Prine out BYTE data in ascending order and with no '0x'(hexa type)
#undef PrintBYTE
#define PrintBYTE(pfile, msg, Data, DataLen) {			\
	int idx;											\
	fprintf(pfile, "%5s =", msg);						\
	for( idx=0; idx<(int)DataLen; idx++) {				\
		if( (idx==0) || ((idx%16)!=0) )					\
			fprintf(pfile, " %.2x", Data[idx]);			\
		else											\
			fprintf(pfile, "\n\t%.2x", Data[idx]);		\
	}													\
	fprintf(pfile, "\n");								\
}

////	ѹ 'ByteLen'-bytes Ÿ óϴ  'Oper'
////	'Iter' ϰ,   KByte/sec .
#define SPEED_TEST(str1, str2, Iter, ByteLen, Oper) {	\
	unsigned idx;										\
	clock_t start, finish;								\
	double Sec, Mbps=0.0;								\
	start = clock();									\
	for( idx=0; idx<Iter; idx++)						\
		{	Oper;	}									\
	finish = clock();									\
	Sec = (double)(finish-start) / CLOCKS_PER_SEC;		\
	if( Sec!=0 )	Mbps = 8.0*ByteLen*idx/Sec/MEGA;	\
	printf("%s%7.3fMbps(=%d*", str1, Mbps, Iter);		\
	printf("%d", ByteLen);								\
	printf("/%3.0f)%s", 1000.0*Sec, str2);				\
}

/*************** New Data Types *******************************************/

/*************** Constant (Error Code) ************************************/
#define CTR_USAGE_ERROR		0x2001

/*************** Global Variables *****************************************/
char	Help[] = "\
Usage1 : -[T/S]   (Test Value/Test Speed)\n\
Usage2 : -[H] infile\n";

/*************** Prototypes ***********************************************/

//########################################################//
//######		Error Code  Լ				######//
//########################################################//

/*************************************************************************/
typedef struct{
	DWORD		ErrorCode;
	BYTE		Message[32];
} ERROR_MESSAGE;

ERROR_MESSAGE	ErrorMessage[] = {
	{CTR_USAGE_ERROR,		"CTR_USAGE_ERROR"},
	{0, ""},
};

/**************************************************************************
*
*/
void	Error(
		DWORD	ErrorCode,
		char	*Message)
{
	DWORD	i;

	for( i=0; ErrorMessage[i].ErrorCode!=0; i++)
		if( ErrorMessage[i].ErrorCode==ErrorCode )	break;

	printf("ERROR(%s) :::: %s\n", ErrorMessage[i].Message, Message);
	if( ErrorCode==CTR_USAGE_ERROR )	printf("\n%s", Help);
	exit(1);
}

//########################################################//
//######		Validity Test  Լ				######//
//########################################################//

/*************************************************************************/
typedef struct{
	DWORD	MsgLen;
	BYTE	Messge[120];
	BYTE	DigestValue[SHA256_DIGEST_VALUELEN];
} TEST_STRUCT;

TEST_STRUCT		TestData[] = {
	////	SHA256
	{	 3, "abc",
		{0xba,0x78,0x16,0xbf,0x8f,0x01,0xcf,0xea,0x41,0x41,0x40,0xde,
		 0x5d,0xae,0x22,0x23,0xb0,0x03,0x61,0xa3,0x96,0x17,0x7a,0x9c,
		 0xb4,0x10,0xff,0x61,0xf2,0x00,0x15,0xad}	},
	{	56, "abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq",
		{0x24,0x8d,0x6a,0x61,0xd2,0x06,0x38,0xb8,0xe5,0xc0,0x26,0x93,
		 0x0c,0x3e,0x60,0x39,0xa3,0x3c,0xe4,0x59,0x64,0xff,0x21,0x67,
		 0xf6,0xec,0xed,0xd4,0x19,0xdb,0x06,0xc1}	},
	{	(DWORD)-1},
};

/**************************************************************************
*
*/
void	ValidityTest()
{
	BYTE	DigestValue[SHA256_DIGEST_VALUELEN];
	DWORD	i, tt;
	SHA256_ALG_INFO	AlgInfo;

	//
	for( i=0; TestData[i].MsgLen!=(DWORD)-1; i++) {
		//
		if( TestData[i].MsgLen!=1000000 ) {
			SHA256_Init(&AlgInfo);
			SHA256_Update(&AlgInfo, TestData[i].Messge, TestData[i].MsgLen);
			SHA256_Final(&AlgInfo, DigestValue);

			//
			fprintf(stdout, "\n==== test %d\n", i);
			PrintBYTE(stdout, "Msg", TestData[i].Messge, TestData[i].MsgLen);
			PrintBYTE(stdout, "DV", DigestValue, SHA256_DIGEST_VALUELEN);
		}
		else {
			SHA256_Init(&AlgInfo);
			for( tt=0; tt<1000000; tt++)
				SHA256_Update(&AlgInfo, TestData[i].Messge, 1);
			SHA256_Final(&AlgInfo, DigestValue);

			//
			fprintf(stdout, "\n==== test %d : 1000000-times 'a'\n", i);
			PrintBYTE(stdout, "DV", DigestValue, SHA256_DIGEST_VALUELEN);
		}
	}
}

/**************************************************************************
*
*/
void	SpeedTest()
{
#define DATA_LEN	1024
	BYTE	Data[DATA_LEN];
	BYTE	DigestValue[SHA256_DIGEST_VALUELEN];
	DWORD	i, tt, Iter;
	SHA256_ALG_INFO	AlgInfo;

	//
	for( tt=0; tt<DATA_LEN; tt++)	Data[tt] = (BYTE) (rand() >> 3);

	//
	for( i=0; i<3; i++) {
		fprintf(stdout, "==== test %d : ", i);
		Iter = 2 * 11500;

		//
		SPEED_TEST("SHA256(1Kbyte data) : ", "\n", Iter, DATA_LEN,
			{	SHA256_Init(&AlgInfo);
				SHA256_Update(&AlgInfo, Data, DATA_LEN);
				SHA256_Final(&AlgInfo, DigestValue);		}	);
	}
#undef DATA_LEN
}

/**************************************************************************
*
*/
void	GeneralTest(
		char	*infile)
{
	FILE	*pIn;
	BYTE	DigestValue[SHA256_DIGEST_VALUELEN];
	BYTE	SrcData[1024+32];
	DWORD	SrcLen;
	SHA256_ALG_INFO	AlgInfo;

	//
	if( (pIn=fopen(infile, "rb"))==NULL ) {
		printf( "The file '%s' was not opened\n", infile);
		Error(0, "File(infile) Open Error");
	}

	//
	SHA256_Init(&AlgInfo);

	for(  ;  ;  ) {
		SrcLen = fread(SrcData, sizeof(BYTE), 1024, pIn);
		if( SrcLen==0 )	break;

		SHA256_Update(&AlgInfo, SrcData, SrcLen);
	}

	SHA256_Final(&AlgInfo, DigestValue);
	PrintBYTE(stdout, "DV", DigestValue, SHA256_DIGEST_VALUELEN);

	//
	fclose(pIn);
}

/**************************************************************************
*
*/
void	ChangeToCapital(char *String)
{
	DWORD	i;

	if( String[0]=='-' ) {
		for( i=1;  ; i++) {
			if( String[i]=='\0' )	break;
			if( (String[i]>='a') && (String[i]<='z') )
				String[i] += 'A' - 'a';
		}
	}
}

/**************************************************************************
*
*/
void	main(int argc, char **argv)
{
	DWORD	i;

	//
	for( i=1; i<(DWORD)argc; i++)
		ChangeToCapital(argv[i]);

	//
	if( argc==2 ) {
		if( strcmp(argv[1], "-T")==0 )		ValidityTest();
		else if( strcmp(argv[1], "-S")==0 )	SpeedTest();
		else	Error(CTR_USAGE_ERROR, "Invalid Use of Argument");
	}
	else if( argc==3 ) {
		//
		if( strcmp(argv[1], "-H")==0 );
		else	Error(CTR_USAGE_ERROR, "Invalid Use of Argument");

		GeneralTest(argv[2]);
	}
	else
		Error(CTR_USAGE_ERROR, "Invalid Number of Argument");
}

/*************** END OF FILE **********************************************/
